﻿using UnityEngine;
using System.Collections;
using System.IO;

public class Noisey : MonoBehaviour {

	int width = 250;
	int length = 250;
	int octaves = 6;
	float horizontalScale = 6f;
	float verticalScale = 200f;
	
	void Start () {
		Vector3[] vertices = new Vector3[width * length];
		Vector2[] uv = new Vector2[width * length];
		int[] triangles = new int[(width - 1) * (length - 1) * 6];

		/* initialize vertices/uv/triangles */
		for (int z = 0; z < width; z++) {
			for (int x = 0; x < length; x++) {
				int index = x + z*width;
				vertices[index] = new Vector3(x,0f,z);
				uv[index] = new Vector2((float)x/(width-1),(float)z/(length-1));
			}
		}
		for (int i = 0; i < triangles.Length; i+=6) {
			int offset = i*width/(6*(width-1));
			triangles[i]   = offset;
			triangles[i+1] = offset+width;
			triangles[i+2] = offset+1;
			triangles[i+3] = offset+1;
			triangles[i+4] = offset+width;
			triangles[i+5] = offset+width+1;
		}
				
		/* do noise */
		for (int i = 0; i < octaves; i++) {
			for (int z = 0; z < width; z++) {
				for (int x = 0; x < length; x++) {
					float frequency = (1<<i)*horizontalScale;
					float amplitude = 1<<(octaves-i);
					float height = Mathf.PerlinNoise (x / frequency, z / frequency) / amplitude;
					vertices[x + z*width].y += height*verticalScale;
				}
			}
		}

		/* apply to gameobject */
		Mesh terrainMesh = new Mesh ();
		terrainMesh.vertices = vertices;
		terrainMesh.uv = uv;
		terrainMesh.triangles = triangles;
		terrainMesh.RecalculateNormals();
		(this.GetComponent<MeshFilter>() as MeshFilter).mesh = terrainMesh;

	}



}
