﻿using UnityEngine;
using System.Collections;

public class PrimAlgorithm : MonoBehaviour {

	public int mazeLength;
	public int mazeWidth;

	public GameObject[,] cells;
	private ArrayList walls;
	private ArrayList cellWalls;

	// Use this for initialization
	void Start () {

		// Initilize cells array and walls list
		int i = 0;
		int j = 0;
		walls = new ArrayList ();
		cellWalls = new ArrayList ();
		cells = new GameObject[mazeLength,mazeWidth];

		// 1. Start with a Grid Full of Walls
		for (i = 0; i < mazeLength; i++) {
			for (j = 0; j < mazeWidth; j++) {
				cells[i,j] = (GameObject) Instantiate(Resources.Load("Cell"));
				cells[i,j].transform.position = new Vector3(i, 0, j);

				Cell currentCell = cells[i,j].GetComponent<Cell>();
				if (i > 0) { Destroy(currentCell.west); currentCell.west = cells[i-1, j].GetComponent<Cell>().east; }
				if (j > 0) { Destroy(currentCell.south); currentCell.south = cells[i, j-1].GetComponent<Cell>().north; }
			}
		}

		// 2. Pick a cell and mark it as part of the maze
		i = Random.Range (2, mazeLength-1);
		j = Random.Range (2, mazeWidth-1);
		i = 1;
		j = 1;
		Cell cell = cells [i, j].GetComponent<Cell> ();
		cell.visited = true;
		GameObject player = (GameObject) Instantiate (Resources.Load ("Player"));
		player.transform.position = new Vector3 (i, 0, j);


		// 3. Add the cell's walls to the wall list
		walls.Add (cell.north);
		walls.Add (cell.south);
		walls.Add (cell.east);
		walls.Add (cell.west);
		cellWalls.Add (cell);
		cellWalls.Add (cell);
		cellWalls.Add (cell);
		cellWalls.Add (cell);
	
		// While there are walls in the list...
		while (walls.Count > 0) {

			// 1. Pick a random wall from the list and a random direction
			int randomPos = Random.Range (0, walls.Count);
			GameObject wall = (GameObject) walls[randomPos];
			Cell wallCell = (Cell) cellWalls[randomPos];

				// Get "cell beyond" position
			i = (int) wallCell.gameObject.transform.position.x;
			j = (int) wallCell.gameObject.transform.position.z;
			if (wallCell.north == wall) { j +=1; }
			else if (wallCell.south == wall) { j -=1; }
			else if (wallCell.east == wall) { i +=1; }
			else if (wallCell.south == wall) { i -=1; }

				// If cell beyond is within maze limits
			if (i > 0 && j > 0 && i < mazeLength && j < mazeWidth) {
				Cell destinationCell = cells[i,j].GetComponent<Cell>();

				// If the cell in that direction isn't in the maze...
				if (!destinationCell.visited) {

					// 1. Make the wall a passage and the new cell part of the maze
					destinationCell.visited = true;
					Destroy (wall);

					// 2. Add the neighboring walls of the cell to the wall list
					if (destinationCell.north != null) { walls.Add (destinationCell.north); cellWalls.Add (destinationCell); }
					if (destinationCell.south != null) { walls.Add (destinationCell.south); cellWalls.Add (destinationCell); }
					if (destinationCell.east != null) { walls.Add (destinationCell.east); cellWalls.Add (destinationCell); }
					if (destinationCell.west != null) { walls.Add (destinationCell.west); cellWalls.Add (destinationCell); }

				}
			}

			// 2. Remove that wall from the list
			walls.RemoveAt(randomPos);
			cellWalls.RemoveAt (randomPos);
		}
	}
}
